@@ -186,38 +186,55 @@ static struct jx *jx_eval_array(struct jx_operator *op, struct jx *left, struct
 static struct jx *jx_eval_call(struct jx *func, struct jx *args, struct jx *ctx) {
 	assert(func);
 	assert(args);
-	assert(args->type == JX_ARRAY);
-
-	if (!jx_istype(func, JX_SYMBOL)) {
-		jx_error(jx_format(
-			"on line %d, unknown function: %s",
-			func->line,
-			func->u.symbol_name
-		));
-	}
+	assert(jx_istype(args, JX_ARRAY));
+	assert(jx_istype(func, JX_SYMBOL));
 
 	if (!strcmp(func->u.symbol_name, "range")) {
-		return jx_function_range(args);
+		return jx_function_range(jx_eval(args, ctx));
 	} else if (!strcmp(func->u.symbol_name, "format")) {
-		return jx_function_format(args);
+		return jx_function_format(jx_eval(args, ctx));
 	} else if (!strcmp(func->u.symbol_name, "join")) {
-		return jx_function_join(args);
+		return jx_function_join(jx_eval(args, ctx));
 	} else if (!strcmp(func->u.symbol_name, "ceil")) {
-		return jx_function_ceil(args);
+		return jx_function_ceil(jx_eval(args, ctx));
 	} else if (!strcmp(func->u.symbol_name, "floor")) {
-		return jx_function_floor(args);
+		return jx_function_floor(jx_eval(args, ctx));
 	} else if (!strcmp(func->u.symbol_name, "basename")) {
-		return jx_function_basename(args);
+		return jx_function_basename(jx_eval(args, ctx));
 	} else if (!strcmp(func->u.symbol_name, "dirname")) {
-		return jx_function_dirname(args);
+		return jx_function_dirname(jx_eval(args, ctx));
 	} else if (!strcmp(func->u.symbol_name, "listdir")) {
-		return jx_function_listdir(args);
+		return jx_function_listdir(jx_eval(args, ctx));
 	} else if (!strcmp(func->u.symbol_name, "escape")) {
-		return jx_function_escape(args);
+		return jx_function_escape(jx_eval(args, ctx));
 	} else if (!strcmp(func->u.symbol_name, "template")) {
-		return jx_function_template(args, ctx);
+		return jx_function_template(jx_eval(args, ctx), ctx);
 	} else if (!strcmp(func->u.symbol_name, "len")) {
-		return jx_function_len(args);
+		return jx_function_len(jx_eval(args, ctx));
+	} else if (!strcmp(func->u.symbol_name, "fetch")) {
+		return jx_function_fetch(jx_eval(args, ctx));
+	} else if (!strcmp(func->u.symbol_name, "schema")) {
+		return jx_function_schema(jx_eval(args, ctx));
+	} else if (!strcmp(func->u.symbol_name, "like")) {
+		return jx_function_like(jx_eval(args, ctx));
+	} else if (!strcmp(func->u.symbol_name, "keys")) {
+		return jx_function_keys(jx_eval(args, ctx));
+	} else if (!strcmp(func->u.symbol_name, "values")) {
+		return jx_function_values(jx_eval(args, ctx));
+	} else if (!strcmp(func->u.symbol_name, "items")) {
+		return jx_function_items(jx_eval(args, ctx));
+
+	/* select() and project() differ from the other functions in
+	 * that they need deferred eval for specific arguments.
+	 * We therefore just give them a copy of the args, and let
+	 * them do the eval themselves.
+	 *
+	 * When adding new functions, you probably don't want to do this
+	 */
+	} else if (!strcmp(func->u.symbol_name, "select")) {
+		return jx_function_select(jx_copy(args), ctx);
+	} else if (!strcmp(func->u.symbol_name, "project")) {
+		return jx_function_project(jx_copy(args), ctx);
 	} else {
 		return jx_error(jx_format(
 			"on line %d, unknown function: %s",
@@ -333,14 +350,7 @@ static struct jx * jx_eval_operator( struct jx_operator *o, struct jx *context )
 	struct jx *right = NULL;
 	struct jx *result = NULL;
 
-	right = jx_eval(o->right,context);
-	if (jx_istype(right, JX_ERROR)) {
-		result = right;
-		right = NULL;
-		goto DONE;
-	}
-
-	if (o->type == JX_OP_CALL) return jx_eval_call(o->left, right, context);
+	if (o->type == JX_OP_CALL) return jx_eval_call(o->left, o->right, context);
 
 	left = jx_eval(o->left,context);
 	if (jx_istype(left, JX_ERROR)) {
@@ -348,6 +358,12 @@ static struct jx * jx_eval_operator( struct jx_operator *o, struct jx *context )
 		left = NULL;
 		goto DONE;
 	}
+	right = jx_eval(o->right,context);
+	if (jx_istype(right, JX_ERROR)) {
+		result = right;
+		right = NULL;
+		goto DONE;
+	}
 
 	if (o->type == JX_OP_SLICE) return jx_operator(JX_OP_SLICE, left, right);
 
@@ -434,6 +450,7 @@ static struct jx_item *jx_eval_comprehension(struct jx *body, struct jx_comprehe
 	struct jx *list = jx_eval(comp->elements, context);
 	if (jx_istype(list, JX_ERROR)) return jx_item(list, NULL);
 	if (!jx_istype(list, JX_ARRAY)) {
+		jx_delete(list);
 		return jx_item(jx_error(jx_format(
 			"on line %d: list comprehension takes an array",
 			comp->line
