@@ -70,11 +70,13 @@ void initialize_units() {
 	rmsummary_add_conversion_field("bytes_written",            "B",       "MB",     "B",       MEGABYTE, 1,             1);
 	rmsummary_add_conversion_field("bytes_received",           "B",       "MB",     "B",       MEGABYTE, 1,             1);
 	rmsummary_add_conversion_field("bytes_sent",               "B",       "MB",     "B",       MEGABYTE, 1,             1);
-	rmsummary_add_conversion_field("bandwidth",                "bps",     "Mbps",   "bps",     1000,     1,             1);
+	rmsummary_add_conversion_field("bandwidth",                "bps",     "Mbps",   "bps",     1000000,  1,             1);
+	rmsummary_add_conversion_field("gpus",                     "gpus",    "gpus",   "gpus",    1,        1,             0);
 	rmsummary_add_conversion_field("cores",                    "cores",   "cores",  "cores",   1,        1,             0);
 	rmsummary_add_conversion_field("cores_avg",                "mcores",  "cores",  "cores",   1000,     1/1000.0,      1);
 	rmsummary_add_conversion_field("machine_cpus",             "cores",   "cores",  "cores",   1,        1,             0);
 	rmsummary_add_conversion_field("machine_load",             "mprocs",  "procs",  "procs",   1000,     1/1000.0,      1);
+	rmsummary_add_conversion_field("context_switches",         "switches","switches","switches", 1,      1,             0);
 	rmsummary_add_conversion_field("max_concurrent_processes", "procs",   "procs",  "procs",   1,        1,             0);
 	rmsummary_add_conversion_field("total_processes",          "procs",   "procs",  "procs",   1,        1,             0);
 	rmsummary_add_conversion_field("total_files",              "files",   "files",  "files",   1,        1,             0);
@@ -254,7 +256,7 @@ int rmsummary_assign_char_field(struct rmsummary *s, const char *key, char *valu
 	return 0;
 }
 
-int64_t rmsummary_get_int_field(struct rmsummary *s, const char *key) {
+int64_t rmsummary_get_int_field(const struct rmsummary *s, const char *key) {
 	if(strcmp(key, "start") == 0) {
 		return s->start;
 	}
@@ -339,6 +341,10 @@ int64_t rmsummary_get_int_field(struct rmsummary *s, const char *key) {
 		return s->cores_avg;
 	}
 
+	if(strcmp(key, "context_switches") == 0) {
+		return s->context_switches;
+	}
+
 	if(strcmp(key, "gpus") == 0) {
 		return s->gpus;
 	}
@@ -361,7 +367,7 @@ int64_t rmsummary_get_int_field(struct rmsummary *s, const char *key) {
 	return 0;
 }
 
-const char *rmsummary_get_char_field(struct rmsummary *s, const char *key) {
+const char *rmsummary_get_char_field(const struct rmsummary *s, const char *key) {
 	if(strcmp(key, "category") == 0) {
 		return s->category;
 	}
@@ -493,6 +499,11 @@ int rmsummary_assign_int_field(struct rmsummary *s, const char *key, int64_t val
 		return 1;
 	}
 
+	if(strcmp(key, "context_switches") == 0) {
+		s->context_switches = value;
+		return 1;
+	}
+
 	if(strcmp(key, "machine_cpus") == 0) {
 		s->machine_cpus = value;
 		return 1;
@@ -590,6 +601,7 @@ struct jx *peak_times_to_json(struct rmsummary *s) {
 	peak_time_to_json(output, cf, s, memory);
 	peak_time_to_json(output, cf, s, total_processes);
 	peak_time_to_json(output, cf, s, max_concurrent_processes);
+	peak_time_to_json(output, cf, s, gpus);
 	peak_time_to_json(output, cf, s, cores);
 	peak_time_to_json(output, cf, s, cpu_time);
 	peak_time_to_json(output, cf, s, machine_cpus);
@@ -643,8 +655,10 @@ struct jx *rmsummary_to_json(const struct rmsummary *s, int only_resources) {
 	field_to_json(output, s, max_concurrent_processes);
 	field_to_json(output, s, cpu_time);
 	field_to_json(output, s, wall_time);
+	field_to_json(output, s, gpus);
 	field_to_json(output, s, cores);
 	field_to_json(output, s, cores_avg);
+	field_to_json(output, s, context_switches);
 	field_to_json(output, s, end);
 	field_to_json(output, s, start);
 
@@ -966,6 +980,8 @@ void rmsummary_read_env_vars(struct rmsummary *s)
 		s->memory = atoi(value);
 	if((value = getenv( RESOURCES_DISK )))
 		s->disk   = atoi(value);
+	if((value = getenv( RESOURCES_GPUS )))
+		s->gpus   = atoi(value);
 }
 
 #define rmsummary_apply_op(dest, src, fn, field) (dest)->field = fn((dest)->field, (src)->field)
@@ -995,8 +1011,10 @@ void rmsummary_bin_op(struct rmsummary *dest, const struct rmsummary *src, rm_bi
 	rmsummary_apply_op(dest, src, fn, total_files);
 	rmsummary_apply_op(dest, src, fn, disk);
 	rmsummary_apply_op(dest, src, fn, fs_nodes);
+	rmsummary_apply_op(dest, src, fn, gpus);
 	rmsummary_apply_op(dest, src, fn, cores);
 	rmsummary_apply_op(dest, src, fn, cores_avg);
+	rmsummary_apply_op(dest, src, fn, context_switches);
 	rmsummary_apply_op(dest, src, fn, machine_cpus);
 	rmsummary_apply_op(dest, src, fn, machine_load);
 
@@ -1091,8 +1109,10 @@ static void merge_limits(struct rmsummary *dest, const struct rmsummary *src)
 	merge_limit(dest, src, bandwidth);
 	merge_limit(dest, src, total_files);
 	merge_limit(dest, src, disk);
+	merge_limit(dest, src, gpus);
 	merge_limit(dest, src, cores);
 	merge_limit(dest, src, cores_avg);
+	merge_limit(dest, src, context_switches);
 	merge_limit(dest, src, machine_load);
 	merge_limit(dest, src, fs_nodes);
 
@@ -1171,7 +1191,9 @@ void rmsummary_merge_max_w_time(struct rmsummary *dest, const struct rmsummary *
 	max_op_w_time(dest, src, bandwidth);
 	max_op_w_time(dest, src, total_files);
 	max_op_w_time(dest, src, disk);
+	max_op_w_time(dest, src, gpus);
 	max_op_w_time(dest, src, cores);
+	max_op_w_time(dest, src, context_switches);
 	max_op_w_time(dest, src, machine_cpus);
 	max_op_w_time(dest, src, machine_load);
 	max_op_w_time(dest, src, fs_nodes);
@@ -1261,6 +1283,10 @@ size_t rmsummary_field_offset(const char *key) {
 		fatal("A field name was not given.");
 	}
 
+	if(!strcmp(key, "gpus")) {
+		return offsetof(struct rmsummary, gpus);
+	}
+
 	if(!strcmp(key, "cores")) {
 		return offsetof(struct rmsummary, cores);
 	}
@@ -1269,6 +1295,10 @@ size_t rmsummary_field_offset(const char *key) {
 		return offsetof(struct rmsummary, cores_avg);
 	}
 
+	if(!strcmp(key, "context_switches")) {
+		return offsetof(struct rmsummary, context_switches);
+	}
+
 	if(!strcmp(key, "disk")) {
 		return offsetof(struct rmsummary, disk);
 	}
@@ -1375,6 +1405,7 @@ int rmsummary_check_limits(struct rmsummary *measured, struct rmsummary *limits)
 	over_limit_check(measured, limits, start);
 	over_limit_check(measured, limits, end);
 	over_limit_check(measured, limits, cores);
+	over_limit_check(measured, limits, gpus);
 	over_limit_check(measured, limits, wall_time);
 	over_limit_check(measured, limits, cpu_time);
 	over_limit_check(measured, limits, max_concurrent_processes);
