@@ -5,9 +5,17 @@ set -ex
 # Find cctools src directory
 CCTOOLS_SRC="$(cd "$(dirname "${BASH_SOURCE[0]}")"/../.. && pwd)"
 
-if [ -z "$DOCKER_IMAGE" ]; then
+if [[ -z "$DOCKER_IMAGE" ]]
+then
     ${CCTOOLS_SRC}/packaging/travis/travis_build.sh
 else
+    if [[ -n "${DOCKER_USERNAME}" ]]
+    then
+        echo "$DOCKER_PASSWORD" | docker login -u "$DOCKER_USERNAME" --password-stdin
+    else
+        echo "Using dockerhub without authentication..."
+    fi
+
     docker run \
         --privileged \
         --ulimit nofile=65536 \
