@@ -176,6 +176,7 @@ FILE  *log_inotify = NULL;      /* List of opened files is written to this file.
 char *template_path = NULL;     /* Prefix of all output files names */
 
 int debug_active = 0;           /* 1 if ACTIVATE_DEBUG_FILE exists. If 1, debug info goes to ACTIVATE_DEBUG_FILE ".log" */
+int enforce_limits = 1;         /* 0 if monitor should only measure, 1 if enforcing resources limits. */
 
 struct jx *verbatim_summary_fields; /* fields added to the summary without change */
 
@@ -951,6 +952,7 @@ void rmonitor_collate_tree(struct rmsummary *tr, struct rmonitor_process_info *p
 
 	/* using .delta here because if we use .accumulated, then we lose information of processes that already terminated. */
 	tr->cpu_time  += p->cpu.delta;
+	tr->context_switches += p->ctx.delta;
 
 	tr->start = summary->start;
 	tr->end   = usecs_since_epoch();
@@ -1001,6 +1003,11 @@ void rmonitor_collate_tree(struct rmsummary *tr, struct rmonitor_process_info *p
 
 	tr->machine_load = p->load.last_minute;
 	tr->machine_cpus = p->load.cpus;
+
+	// hack: set gpu limit as the measured gpus:
+	if(resources_limits->gpus > 0) {
+		tr->gpus = resources_limits->gpus;
+	}
 }
 
 void rmonitor_find_max_tree(struct rmsummary *result, struct rmsummary *tr)
@@ -1135,9 +1142,14 @@ void decode_zombie_status(struct rmsummary *summary, int wait_status)
 
 	if(summary->limits_exceeded)
 	{
+		/* record that limits were exceeded in the summary, but only change the
+		 * exit_status when enforcing limits. */
 		free(summary->exit_type);
 		summary->exit_type   = xxstrdup("limits");
-		summary->exit_status = 128 + SIGTERM;
+
+		if(enforce_limits) {
+			summary->exit_status = 128 + SIGTERM;
+		}
 	}
 }
 
@@ -1288,7 +1300,7 @@ int rmonitor_final_summary()
 
 	int status;
 
-	if(summary->limits_exceeded) {
+	if(summary->limits_exceeded && enforce_limits) {
 		status = RM_OVERFLOW;
 	} else if(summary->exit_status != 0) {
 		status = RM_TASK_ERROR;
@@ -1818,8 +1830,9 @@ int rmonitor_dispatch_msg(void)
 
 	summary->last_error = msg.error;
 
-	if(!rmsummary_check_limits(summary, resources_limits))
+	if(!rmsummary_check_limits(summary, resources_limits) && enforce_limits) {
 		rmonitor_final_cleanup(SIGTERM);
+	}
 
 	// find out if messages are urgent:
 	if(msg.type == SNAPSHOT) {
@@ -2006,6 +2019,10 @@ struct rmonitor_process_info *spawn_first_process(const char *executable, char *
 
         debug(D_RMON, "executing: %s\n", executable);
 
+		char *pid_s = string_format("%d", getpid());
+		setenv(RESOURCE_MONITOR_ROOT_PROCESS, pid_s, 1);
+		free(pid_s);
+
 		errno = 0;
         execvp(executable, argv);
         //We get here only if execlp fails.
@@ -2024,7 +2041,7 @@ static void show_help(const char *cmd)
 {
     fprintf(stdout, "\nUse: %s [options] -- command-line-and-options\n\n", cmd);
     fprintf(stdout, "%-30s Enable debugging for this subsystem.\n", "-d,--debug=<subsystem>");
-	fprintf(stdout, "%-30s Send debugging to this file. (can also be :stderr, :stdout, :syslog, or :journal)\n", "-o,--debug-file=<file>");
+	fprintf(stdout, "%-30s Send debugging to this file. (can also be :stderr, or :stdout)\n", "-o,--debug-file=<file>");
     fprintf(stdout, "%-30s Show this message.\n", "-h,--help");
     fprintf(stdout, "%-30s Show version string.\n", "-v,--version");
     fprintf(stdout, "\n");
@@ -2036,6 +2053,7 @@ static void show_help(const char *cmd)
     fprintf(stdout, "%-30s Use maxfile with list of var: value pairs for resource limits.\n", "-l,--limits-file=<maxfile>");
     fprintf(stdout, "%-30s Use string of the form \"var: value, var: value\" to specify.\n", "-L,--limits=<string>");
     fprintf(stdout, "%-30s resource limits. Can be specified multiple times.\n", "");
+    fprintf(stdout, "%-30s Do not enforce resource limits, only measure resources.\n", "--measure-only");
     fprintf(stdout, "\n");
     fprintf(stdout, "%-30s Keep the monitored process in foreground (for interactive use).\n", "-f,--child-in-foreground");
     fprintf(stdout, "\n");
@@ -2101,7 +2119,7 @@ int rmonitor_resources(long int interval /*in seconds */)
 		rmonitor_find_max_tree(snapshot, resources_now);
 		rmonitor_log_row(resources_now);
 
-		if(!rmsummary_check_limits(summary, resources_limits)) {
+		if(!rmsummary_check_limits(summary, resources_limits) && enforce_limits) {
 			rmonitor_final_cleanup(SIGTERM);
 		}
 
@@ -2208,7 +2226,8 @@ int main(int argc, char **argv) {
 		LONG_OPT_CATALOG_SERVER,
 		LONG_OPT_CATALOG_PROJECT,
 		LONG_OPT_CATALOG_INTERVAL,
-		LONG_OPT_PID
+		LONG_OPT_PID,
+		LONG_OPT_MEASURE_ONLY
 	};
 
     static const struct option long_options[] =
@@ -2223,6 +2242,7 @@ int main(int argc, char **argv) {
 		    {"limits-file",required_argument, 0, 'l'},
 		    {"sh",         required_argument, 0, 'c'},
 		    {"pid",        required_argument, 0, LONG_OPT_PID},
+		    {"measure-only", no_argument, 0, LONG_OPT_MEASURE_ONLY},
 
 		    {"verbatim-to-summary",required_argument, 0, 'V'},
 
@@ -2343,6 +2363,9 @@ int main(int argc, char **argv) {
 					first_process_pid = (pid_t) p;
 				}
 				break;
+			case LONG_OPT_MEASURE_ONLY:
+				enforce_limits = 0;
+				break;
 			case LONG_OPT_CATALOG_TASK_READABLE_NAME:
 				catalog_task_readable_name = xxstrdup(optarg);
 				break;
