@@ -66,19 +66,31 @@ static struct jx_table queue_headers[] = {
 };
 
 static struct jx_table task_headers[] = {
-{"taskid",       "ID",      JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 8},
+{"taskid",       "ID",      JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 4},
 {"state",        "STATE",   JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,  8},
 {"priority",     "PRIORITY",JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_RIGHT, 8},
-{"host",         "HOST",    JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, -24},
-{"command",      "COMMAND", JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, -30},
+{"host",         "HOST",    JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, -10},
+{"command",      "COMMAND", JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, -18},
+{"cores",        "CORES",   JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 8},
+{"memory",       "MEMORY",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 8},
+{"disk",         "DISK",    JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 8},
+{"gpus",         "GPUS",    JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 8},
 {NULL,NULL,0,0,0}
 };
 
 static struct jx_table worker_headers[] = {
-{"hostname",            "HOST",     JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, -24},
-{"address_port",        "ADDRESS",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,16},
-{"total_tasks_complete","COMPLETED",JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_RIGHT, 9},
-{"total_tasks_running", "RUNNING",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,8},
+{"hostname",            "HOST",      JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-10},
+{"address_port",        "ADDRESS",   JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-15},
+{"total_tasks_complete","COMPLETED", JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_RIGHT,9},
+{"total_tasks_running", "RUNNING",   JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,7},
+{"cores_inuse",         "CORE_USE",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-6},
+{"cores_total",         "CORE_ALL",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-6},
+{"memory_inuse",        "MEM_USE",   JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-5},
+{"memory_total",        "MEM_ALL",   JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-5},
+{"disk_inuse",          "DISK_USE",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-6},
+{"disk_total",          "DISK_ALL",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-6},
+{"gpus_inuse",          "GPUS_USE",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-6},
+{"gpus_total",          "GPUS_ALL",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT,-6},
 {NULL,NULL,0,0,0}
 };
 
@@ -93,16 +105,19 @@ static struct jx_table workers_able_headers[] = {
 {NULL,NULL,0,0,0}
 };
 
-static struct jx_table master_resource_headers[] = {
-{"project",     "MASTER", JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 30},
-{"cores_total", "CORES",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 10},
-{"memory_total","MEMORY", JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 15},
-{"disk_total",  "DISK",   JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 20},
+static struct jx_table manager_resource_headers[] = {
+{"project",			"MANAGER", 		JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, -19},
+{"cores_total",		"CORES",        JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_RIGHT, 6},
+{"cores_inuse",		"INUSE", 	    JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_RIGHT, 6},
+{"memory_total",	"MEM(GB)",     JX_TABLE_MODE_GIGABYTES, JX_TABLE_ALIGN_RIGHT, 8},
+{"memory_inuse",	"INUSE", 	JX_TABLE_MODE_GIGABYTES, JX_TABLE_ALIGN_RIGHT, 8},
+{"disk_total",  	"DISK(GB)",  	JX_TABLE_MODE_GIGABYTES, JX_TABLE_ALIGN_RIGHT, 9},
+{"disk_inuse",		"INUSE", 	JX_TABLE_MODE_GIGABYTES, JX_TABLE_ALIGN_RIGHT, 9},
 {NULL,NULL,0,0,0}
 };
 
 static struct jx_table capacity_headers[] = {
-{"project",     "MASTER", JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 30},
+{"project",     "MANAGER", JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 30},
 {"capacity_tasks", "TASKS",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 10},
 {"capacity_cores", "CORES",  JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 10},
 {"capacity_memory","MEMORY", JX_TABLE_MODE_PLAIN, JX_TABLE_ALIGN_LEFT, 15},
@@ -112,25 +127,25 @@ static struct jx_table capacity_headers[] = {
 
 static void show_help(const char *progname)
 {
-	fprintf(stdout, "usage: %s [master] [port]\n", progname);
-	fprintf(stdout, "If a master and port are given, get data directly from that master.\n");
+	fprintf(stdout, "usage: %s [manager] [port]\n", progname);
+	fprintf(stdout, "If a manager and port are given, get data directly from that manager.\n");
 	fprintf(stdout, "Otherwise, contact the catalog server for summary data.\n");
 	fprintf(stdout, "Options:\n");
 	fprintf(stdout, " %-30s Show queue summary statistics. (default)\n", "-Q,--statistics");
-	fprintf(stdout, " %-30s Filter results of -Q for masters matching <name>\n", "-M,--project-name<name>");
-	fprintf(stdout, " %-30s List workers connected to the given master.\n", "-W,--workers");
-	fprintf(stdout, " %-30s List tasks of the given master.\n", "-T,--tasks");
-	fprintf(stdout, " %-30s List categories of the given master, size of\n", "-A,--able-workers");
+	fprintf(stdout, " %-30s Filter results of -Q for managers matching <name>\n", "-M,--project-name<name>");
+	fprintf(stdout, " %-30s List workers connected to the given manager.\n", "-W,--workers");
+	fprintf(stdout, " %-30s List tasks of the given manager.\n", "-T,--tasks");
+	fprintf(stdout, " %-30s List categories of the given manager, size of\n", "-A,--able-workers");
 	fprintf(stdout, " %-30s largest task, and workers that can run it.\n", "");
-	fprintf(stdout, " %-30s Shows aggregated resources of all masters.\n", "-R,--resources");
-	fprintf(stdout, " %-30s Shows resource capacities of all masters.\n", "   --capacity");
+	fprintf(stdout, " %-30s Shows aggregated resources of all managers.\n", "-R,--resources");
+	fprintf(stdout, " %-30s Shows resource capacities of all managers.\n", "   --capacity");
 	fprintf(stdout, " %-30s Long text output.\n", "-l,--verbose");
 	fprintf(stdout, " %-30s Set catalog server to <catalog>. Format: HOSTNAME:PORT\n", "-C,--catalog=<catalog>");
 	fprintf(stdout, " %-30s Enable debugging for this subsystem.\n", "-d,--debug <flag>");
 	fprintf(stdout, " %-30s Filter results by this expression.\n", "   --where=<expr>");
 	fprintf(stdout, " %-30s RPC timeout (default is %ds).\n", "-t,--timeout=<time>", work_queue_status_timeout);
 	fprintf(stdout, " %-30s Send debugging to this file. (can also be :stderr,\n", "-o,--debug-file=<file>");
-	fprintf(stdout, " %-30s :stdout, :syslog, or :journal)\n", "");
+	fprintf(stdout, " %-30s or :stdout)\n", "");
 	fprintf(stdout, " %-30s Rotate debug file once it reaches this size.\n", "-O,--debug-rotate-max=<bytes>");
 	fprintf(stdout, " %-30s Show work_queue_status version.\n", "-v,--version");
 	fprintf(stdout, " %-30s This message.\n", "-h,--help");
@@ -141,7 +156,7 @@ enum {
 	LONG_OPT_CAPACITY
 };
 
-static void work_queue_status_parse_command_line_arguments(int argc, char *argv[], const char **master_host, int *master_port, const char **project_name)
+static void work_queue_status_parse_command_line_arguments(int argc, char *argv[], const char **manager_host, int *manager_port, const char **project_name)
 {
 	static const struct option long_options[] = {
 		{"project-name", required_argument, 0, 'M'},
@@ -163,7 +178,7 @@ static void work_queue_status_parse_command_line_arguments(int argc, char *argv[
 		{0,0,0,0}};
 
 	signed int c;
-	int needs_explicit_master = 0;
+	int needs_explicit_manager = 0;
 
 	while((c = getopt_long(argc, argv, "AM:QTWC:d:lo:O:Rt:vh", long_options, NULL)) > -1) {
 		switch (c) {
@@ -179,25 +194,25 @@ static void work_queue_status_parse_command_line_arguments(int argc, char *argv[
 		case 'Q':
 			if(query_mode != NO_QUERY)
 				fatal("Options -A, -Q, -T, and -W, are mutually exclusive, and can be specified only once.");
-			needs_explicit_master = 0;
+			needs_explicit_manager = 0;
 			query_mode = QUERY_QUEUE;
 			break;
 		case 'T':
 			if(query_mode != NO_QUERY)
 				fatal("Options -A, -Q, -T, and -W, are mutually exclusive, and can be specified only once.");
-			needs_explicit_master = 1;
+			needs_explicit_manager = 1;
 			query_mode = QUERY_TASKS;
 			break;
 		case 'W':
 			if(query_mode != NO_QUERY)
 				fatal("Options -A, -Q, -T, and -W, are mutually exclusive, and can be specified only once.");
-			needs_explicit_master = 1;
+			needs_explicit_manager = 1;
 			query_mode = QUERY_WORKERS;
 			break;
 		case 'A':
 			if(query_mode != NO_QUERY)
 				fatal("Options -A, -Q, -T, and -W, are mutually exclusive, and can be specified only once.");
-			needs_explicit_master = 1;
+			needs_explicit_manager = 1;
 			query_mode = QUERY_ABLE_WORKERS;
 			break;
 		case 'l':
@@ -246,19 +261,19 @@ static void work_queue_status_parse_command_line_arguments(int argc, char *argv[
 	if(query_mode == NO_QUERY)
 		query_mode = QUERY_QUEUE;
 
-	if(needs_explicit_master && optind >= argc)
-		fatal("Options -A, -T and -W need an explicit master to query.");
+	if(needs_explicit_manager && optind >= argc)
+		fatal("Options -A, -T and -W need an explicit manager to query.");
 
 	if(*project_name && query_mode != QUERY_QUEUE)
 		fatal("Option -M,--project-name can only be used together with -Q,--statistics");
 
 	if( optind < argc ) {
-		*master_host = argv[optind];
+		*manager_host = argv[optind];
 		optind++;
 	}
 
 	if( optind < argc ) {
-		*master_port = atoi(argv[optind]);
+		*manager_port = atoi(argv[optind]);
 		optind++;
 	}
 
@@ -281,7 +296,7 @@ void resize_catalog(size_t new_size)
 	catalog_size = new_size;
 }
 
-int get_masters( time_t stoptime )
+int get_managers( time_t stoptime )
 {
 	struct catalog_query *cq;
 	struct jx *j;
@@ -295,11 +310,17 @@ int get_masters( time_t stoptime )
 		JX_OP_AND,
 		jexpr,
 		jx_operator(
-			JX_OP_EQ,
-			jx_symbol("type"),
-			jx_string("wq_master")
-		)
-	);
+			JX_OP_OR,
+				jx_operator(
+					JX_OP_EQ,
+					jx_symbol("type"),
+					jx_string("wq_manager")
+				),
+				jx_operator(
+					JX_OP_EQ,
+					jx_symbol("type"),
+					jx_string("wq_master")
+				)));
 
 	cq = catalog_query_create(catalog_host, jexpr, stoptime );
 	if(!cq)
@@ -363,9 +384,9 @@ int find_child_relations(int spaces, const char *host, int port, struct jx_table
 
 	while(global_catalog[i] != NULL)
 	{
-		const char *temp_my_master = jx_lookup_string(global_catalog[i], "my_master");
+		const char *temp_my_manager = jx_lookup_string(global_catalog[i], "my_manager");
 
-		if(temp_my_master && !strcmp(temp_my_master, full_address))
+		if(temp_my_manager && !strcmp(temp_my_manager, full_address))
 		{
 			struct jx *jproject = jx_lookup(global_catalog[i],"project");
 			const char *project_name = jproject->u.string_value;
@@ -415,8 +436,8 @@ int do_catalog_query(const char *project_name, struct jx_table *headers, time_t
 			}
 			jx_print_stream(global_catalog[i], stdout);
 		} else if(format_mode==FORMAT_TABLE) {
-			const char *temp_my_master = jx_lookup_string(global_catalog[i], "my_master");
-			if( !temp_my_master || !strcmp(temp_my_master, "127.0.0.1:-1") ) { //this master has no master
+			const char *temp_my_manager = jx_lookup_string(global_catalog[i], "my_manager");
+			if( !temp_my_manager || !strcmp(temp_my_manager, "127.0.0.1:-1") ) { //this manager has no manager
 
 				if(!project_name || whole_string_match_regex(jx_lookup_string(global_catalog[i], "project"), project_name)) {
 					jx_table_print(headers,global_catalog[i], stdout, columns);
@@ -440,9 +461,9 @@ int do_catalog_query(const char *project_name, struct jx_table *headers, time_t
 	return EXIT_SUCCESS;
 }
 
-int do_direct_query( const char *master_host, int master_port, time_t stoptime )
+int do_direct_query( const char *manager_host, int manager_port, time_t stoptime )
 {
-	static struct jx_table *query_headers[] = { [QUERY_QUEUE] = queue_headers, task_headers, worker_headers, workers_able_headers, master_resource_headers };
+	static struct jx_table *query_headers[] = { [QUERY_QUEUE] = queue_headers, task_headers, worker_headers, workers_able_headers, manager_resource_headers };
 	static const char * query_strings[] = { [QUERY_QUEUE] = "queue","task","worker", "wable", "resources"};
 
 	struct jx_table *query_header = query_headers[query_mode];
@@ -450,16 +471,16 @@ int do_direct_query( const char *master_host, int master_port, time_t stoptime )
 
 	struct link *l;
 
-	char master_addr[LINK_ADDRESS_MAX];
+	char manager_addr[LINK_ADDRESS_MAX];
 
-	if(!domain_name_cache_lookup(master_host,master_addr)) {
-		fprintf(stderr,"couldn't find address of %s\n",master_host);
+	if(!domain_name_cache_lookup(manager_host,manager_addr)) {
+		fprintf(stderr,"couldn't find address of %s\n",manager_host);
 		return 1;
 	}
 
-	l = link_connect(master_addr,master_port,stoptime);
+	l = link_connect(manager_addr,manager_port,stoptime);
 	if(!l) {
-		fprintf(stderr,"couldn't connect to %s port %d: %s\n",master_host,master_port,strerror(errno));
+		fprintf(stderr,"couldn't connect to %s port %d: %s\n",manager_host,manager_port,strerror(errno));
 		return 1;
 	}
 
@@ -468,7 +489,7 @@ int do_direct_query( const char *master_host, int master_port, time_t stoptime )
 	struct jx *jarray = jx_parse_link(l,stoptime);
 
 	if(!jarray || jarray->type != JX_ARRAY) {
-		fprintf(stderr,"couldn't read from %s port %d: %s",master_host,master_port,strerror(errno));
+		fprintf(stderr,"couldn't read from %s port %d: %s",manager_host,manager_port,strerror(errno));
 		return 1;
 	}
 
@@ -482,12 +503,7 @@ int do_direct_query( const char *master_host, int master_port, time_t stoptime )
 			}
 		}
 	} else {
-		if( query_mode == QUERY_QUEUE ) {
-			//if queue info, drop the array.
-			jx_print_stream(jarray->u.items->value,stdout);
-		} else {
-			jx_print_stream(jarray,stdout);
-		}
+		jx_print_stream(jarray,stdout);
 	}
 
 	jx_delete(jarray);
@@ -519,13 +535,13 @@ int terminal_columns( int fd )
 
 int main(int argc, char *argv[])
 {
-	const char *master_host  = NULL;
+	const char *manager_host  = NULL;
 	const char *project_name = NULL;
-	int master_port = WORK_QUEUE_DEFAULT_PORT;
+	int manager_port = WORK_QUEUE_DEFAULT_PORT;
 
 	debug_config(argv[0]);
 
-	work_queue_status_parse_command_line_arguments(argc, argv, &master_host, &master_port, &project_name);
+	work_queue_status_parse_command_line_arguments(argc, argv, &manager_host, &manager_port, &project_name);
 
 	cctools_version_debug(D_DEBUG, argv[0]);
 
@@ -533,14 +549,14 @@ int main(int argc, char *argv[])
 
 	time_t stoptime = time(0) + work_queue_status_timeout;
 
-	if(master_host) {
-		return do_direct_query(master_host,master_port,stoptime);
+	if(manager_host) {
+		return do_direct_query(manager_host,manager_port,stoptime);
 	} else {
 		global_catalog = malloc(sizeof(*global_catalog)*CATALOG_SIZE); //only malloc if catalog queries are being done
-		get_masters(stoptime);
+		get_managers(stoptime);
 		struct jx_table *h;
 		if(query_mode==QUERY_MASTER_RESOURCES) {
-			h = master_resource_headers;
+			h = manager_resource_headers;
 		} else if(query_mode==QUERY_CAPACITIES) {
 			h = capacity_headers;
 		}
